#ifndef _bicoti_actor_basic
#define _bicoti_actor_basic

#include <iostream.h>
#include <source/event_loop_manager/event_loop_manager_basic.hpp>

//--------------------DECLARATIONS---------------------
//------------Definiciones de actores estandar---------
/** 
 * Tipos de actores
 *
 *<pre>
 *  Tipo           Descripcin
 *  ========       =======================
 *  _graphic       actor grfico
 *  _timer         actor timer
 *</pre>
 */ 
typedef enum { _graphic, _timer } ActorTypes;

/**
 * Dimensiones
 *
 *<pre>
 *  Tipo          Descripcin
 *  ======        ===========================
 *  _none         actor adimesional
 *  _2D           actor en 2 dimensiones
 *  _3D           actor en 3 dimensiones
 *  _menu         caso particular de 2D
 *</pre>
 */ 
typedef enum { _none, _2D, _3D, _menu } DimensionCases;

/**
 * Clases de Actores
 *
 *<pre>
 *  Clase                      Descripcin (actor)
 *  ========                   ==================
 *  actor_panel                 panel ( bicotiActorGraphic2DPanel )
 *  actor_image                 imagen  ( bicotiActorGraphic2DImage < PixelType > )
 *  actor_push_button           botn  ( bicotiActorGraphic2DPushButton )
 *  actor_label                 etiqueta ( bicotiActorGraphic2DLabel )
 *  actor_timer                 timer ( bicotiActorTimer ) 
 *  actor_field                 campo ( bicotiActorGraphic2DField )
 *  actor_menu_popup            men desplegable ( bicotiActorGraphicMenuPopup )
 *  actor_menu_bar              barra de men ( bicotiActorGraphicMenuBar )
 *  actor_file_dialog           dilogo de archivo  ( bicotiActorGraphic2DFileDialog )
 *  actor_scroll_bar            scroll  ( bicotiActorGraphic2DScrollBar )
 *</pre>
 */
typedef enum { actor_panel, actor_image, actor_push_button, actor_label, actor_timer, actor_field, actor_menu_popup, actor_menu_bar, actor_file_dialog, actor_scroll_bar } ActorCases;


//----------------bicotiActorDescriptor----------------

/**
 * Descriptor de actor
 *
 * Encapsula informacin sobre el actor
 */
class bicotiActorDescriptor 
{
private:
  char * actor_name;
  ActorCases actor_case;
  DimensionCases dimension_case;
  ActorTypes actor_type;
  bool main_actor;

public:
  
  /**
   * Constructor
   *
   * Construye una descriptor del actor
   *@param act_name nombre del actor
   *@param act_type tipo de actor
   *@param act_dim dimension del actor
   *@param act_case clase del actor
   */
  bicotiActorDescriptor( char * act_name, ActorTypes act_type, DimensionCases act_dim, ActorCases act_case );
  
  /**
   * Asignacin
   *
   * Copia los datos de un  descriptor de  actor en el nuevo actor
   *@param actor descriptor de actor original
   */
  void operator=( const bicotiActorDescriptor& );

  /** 
   * Devuelve el nombre del actor 
   *
   *@returns nombre del actor
   */
  char * GetActorName();
  
  /**
   * Devuelve la clase del actor
   *
   *@returns clase del actor
   *@see ActorCases 
   */
  ActorCases GetActorCase();
  
  /**
   * Devuelve la dimensin del actor
   *
   *@returns dimensin del actor ( ningun, 2D, 3D, etc.)
   *@see DimensionCases
   */
  DimensionCases GetActorDimension();
  
  /**
   * Devuelve el tipo del actor
   *
   *@returns tipo de actor ( graphic, timer )
   *@see ActorTypes
   */
  ActorTypes GetActorType();
  
  /**
   * Indica si el actor es un actor principal 
   *  
   * Indica si el actor es un actor principal ( controla el event_loop )
   *@returns true si es actor principal
   */
  bool IsMainActor();
};


//---------------------bicotiActor---------------------
class bicotiHomologous;


/**
 * Clase base para todos los actores de bicoti
 *
 *@see bicotiActorGraphic
 *@see bicotiActorGraphic2D
 *@see bicotiActorTimer
 */
class bicotiActor
{
  friend class bicotiEventLoopManager; 
  
protected:
  char * actor_name;
  bool main_actor;
  ActorTypes actor_type;
  DimensionCases dimension_case;
  ActorCases actor_case;

  bicotiHomologous * ptr_homologous;
  static bicotiEventLoopManager * ptr_current_event_loop;

public:
 
  /**
   *  Constructor
   *
   * Construye un nuevo bicotiActor
   *@param act_name nombre del actor
   */
  bicotiActor( const char * act_name = NULL );
  
  /** Destructor */
  virtual ~bicotiActor();

  /** 
   * Devuelve el descriptor del actor
   *
   *@returns el descriptor del actor
   */
  bicotiActorDescriptor GetActorDescriptor();

  /**
   * Asocia un homlogo al actor
   *
   *@param ptr_homo homlogo
   */
  void SetHomologous( bicotiHomologous * ptr_homo );
  
  /** 
   * Devuelve un puntero al homologo 
   *
   *@returns puntero al homologo del actor
   */
  bicotiHomologous * GetHomologous();  
  
  /** Borra el homlogo */
  virtual void DestroyHomologous()=0;

  /** 
   * Devuelve un puntero al event loop actual
   *
   *@returns puntero al event loop
   */
  bicotiEventLoopManager * GetCurrentEventLoop();
};


//------------------bicotiHomologous-------------------

/** 
 * Homlogo de bicotiActor
 *
 */
class bicotiHomologous
{
protected:
  bicotiActor * ptr_actor;
  
public:

  /** Destructor */
  virtual ~bicotiHomologous();

  /**
   * Asocia el actor al homologo
   *
   *@param ptr_source actor
   */
  void SetActor( bicotiActor * ptr_source );
  
  /** 
   * Devuelve un puntero al actor
   *
   *@returns puntero al actor asociado
   */
  bicotiActor * GetActor();
};


#endif
