#ifndef _bicoti_actor_graphic_2D
#define _bicoti_actor_graphic_2D


#include <source/actor/actor_graphic.hpp>
#include <source/interaction/interaction_geometry.hpp>


//--------------------DECLARATIONS---------------------
//----------------bicotiActorGraphic2D-----------------

/**
 * Clase base para todos los actores grficos 2D
 *
 * Todos los actores grficos 2D comparten las propiedades de tener una representacin grfica en pantalla y que sus dimensiones y posicin dentro de la misma pueden ser fijadas por el usuario. Son fuente de eventos geomtricos.
 *@see bicotiHomologousGraphic2D
 *@see bicotiActorGraphic2DField 
 *@see bicotiActorGraphic2DFileDialog 
 *@see bicotiActorGraphic2DImage
 *@see bicotiActorGraphic2DLabel 
 *@see bicotiActorGraphic2DPanel 
 *@see bicotiActorGraphic2DPushButton 
 *@see bicotiActorGraphic2DScrollBar 
*/
class bicotiActorGraphic2D : public bicotiActorGraphic, public bicotiInteractionSourceGeometry
{
protected:
  int x, y;
  int width, height;
  int min_width, max_width, min_height, max_height;

  /**
   * Verifica si cambi la posicin del actor 
   *
   * Verifica si cambi la posicin del actor y actualiza al actor con la nueva posicin. Reimplementa el mtodo de bicotiInteractionSourceGeometry.
   *@param x_in coordenada horizontal de la nueva posicin
   *@param y_in coordenada vertical de la nueva posicin
   */
  bool CheckPosition( int x_in, int y_in);
 
  /**
   * Verifica si cambi el tamao del actor 
   *
   * Verifica si cambi el tamao del actor y actualiza al actor con el nuevo tamao. Reimplementa el mtodo de bicotiInteractionSourceGeometry.
   *@param w nuevo ancho
   *@param h nueva altura
   */
  bool CheckSize( int w, int h );
 
public:

  /** 
   * Constructor
   *
   * Construye un nuevo bicotiActorGraphic2D
   *@param act_name nombre del actor
   */
  bicotiActorGraphic2D( char * act_name );
  
  /** Destructor */
  virtual ~bicotiActorGraphic2D();

  /**
   * Fija la posicion del actor
   *
   * Fija la posicion del actor relativa a la pantalla o a su padre si lo tiene.
   *@param x_in coordenada horizontal
   *@param y_in coordenada vertical
   */
  void SetPosition( int x_in, int y_in);
  
  /**
   * Fija la coordenada horizontal del actor
   *
   * Fija la coordenada horizontal del actor relativa a la pantalla o a su padre si tiene.
   *@param x_in coordenada horizontal
   */
  void SetX( int x_in );
  /** 
   * Fija la coordenada vertical del actor
   *
   * Fija la coordenada vertical del actor relativa a la pantalla o a su padre si lo tiene
   *@param y_in coordenda vertical
   */
  void SetY( int y_in  );
  
  /**
   * Fija el tamao del actor
   *
   *@param w ancho
   *@param h altura
   */
  void SetSize( int w, int h );
  
  /**
   * Fija el ancho del actor
   *
   *@param w ancho del actor
   */
  void SetWidth( int w );
 
  /**
   * Fija la altura 
   *
   *@param h altura del actor
   */
  void SetHeight( int h );
  
  /**
   * Devuelve la coordenada horizontal del actor
   *
   * Devuelve la coordenada horizontal del actor relativa a la pantalla o su padre si lo tiene.
   *@returns coordenada horizontal
   *@see #SetX
   */
  int GetX();
  
  /**
   * Devuelve la coordenada vertical del actor
   *
   * Devuelve la coordenda vertical del actor relativa a la pantalla o a su padre si lo tiene
   *@returns coordenada vertical
   *@see #SetY
   */
  int GetY();
  
  /** 
   * Devuelve el ancho del actor
   *
   *@returns ancho del actor
   *@see #SetWidth
   */ 
  int GetWidth();

  /**
   * Devuelve la altura del actor
   *
   *@returns altura del actor
   *@see #SetHeight
   */
  int GetHeight();

  /**
   * Devuelve ancho mnimo del actor
   *
   *@returns ancho mnimo del actor
   *@see #SetMinimumWidth
   */
  int GetMinimumWidth();
  
  /**
   * Devuelve ancho mximo del actor
   *
   *@returns ancho mximo del actor
   *@see #SetMaximumWidth
   */
  int GetMaximumWidth();
  
  /**
   * Devuelve altura mnima del actor
   *
   *@returns altura mnima del actor
   *@see #SetMinimumHeight
   */
  int GetMinimumHeight();

  /**
   * Devuelve altura mxima del actor
   *
   *@returns altura mxima del actor
   *@see #SetMaximumHeight
   */
  int GetMaximumHeight();
  
  /**
   * Fija ancho mnimo el actor
   *
   * No podr reducirse la ventana a menos del ancho especificado
   *@param min_w ancho mnimo
   */
  void SetMinimumWidth( int min_w );

  /**
   * Fija el ancho mximo del actor
   *
   * No podra agrandarse la ventana a ms del ancho espedicificado
   *@param max_w ancho mximo
   */
  void SetMaximumWidth( int max_w );
 
  /** 
   * Fija altura mnima del actor
   * 
   * No podr reducirse la ventana a menos de la altura especificada
   *@param mim_h ancho altura mnima
   */
  void SetMinimumHeight( int min_h );

  /**
   * Fija la altura mxima del actor
   *
   * No podr agrandarse la ventana a ms de la altura especificada
   *@param max_h altura mxima
   */
  void SetMaximumHeight( int max_h);
 
  /**
   * Fija el ancho a un valor constante
   *
   * No se puede modificar el ancho del actor
   *@param w ancho fijo
   */
  void SetFixedWidth( int w);
  
  /**
   * Fija la altura a un valor constante
   *
   * No se puede modificar la altura del actor
   *@param h altura fija
   */
  void SetFixedHeight( int h);
  
  /**
   * Fija el tamao a un valor constante
   *
   * No se puede modificar el tamao del actor
   *@param w ancho fijo
   *@param h altura fija
   */
  void SetFixedSize( int w, int h);
};


//-----------------bicotiHomologous2D------------------

/**
 * Homologo para bicotiActorGraphic2D
 *
 *@see bicotiActorGraphic2D
 */
class bicotiHomologousGraphic2D : public bicotiHomologousGraphic
{
public:

  /** Destructor */
  virtual ~bicotiHomologousGraphic2D();

  /**
   * Fija la posicin del actor
   *
   *@param x_in coordenada horizontal
   *@param y_in coordenada vertical
   *@see bicotiActorGraphic2D::SetPosition
   */
  virtual void SetPosition( int x_in, int y_in )=0;

  /**
   * Fija el tamao del actor
   *
   *@param w ancho del actor
   *@param h altura del actor 
   *@see bicotiActorGraphic2D::SetSize
   */
  virtual void SetSize( int w, int h )=0;

  /**
   * Fija ancho mnima del actor
   *
   *@param w ancho mnimo del actor
   *@see bicotiActorGraphic2D::SetMinimumWidth
   */
  virtual void SetMinimumWidth( int w )=0;
  
  /**
   * Fija ancho mximo del actor
   *
   *@param w ancho mximo del actor
   *@see bicotiActorGraphic2D::SetMaximumWidth
   */
  virtual void SetMaximumWidth( int w )=0;
  
  /**
   * Fija altura mnima del actor
   *
   *@param altura mnima del actor
   *@see bicotiActorGraphic2D::SetMinimumHeight
   */
  virtual void SetMinimumHeight( int h )=0;
 
  /**
   * Fija altura mxima del actor
   *
   *@param h altura mxima del actor
   *@see bicotiActorGraphic2D::SetMaximumHeight
   */
  virtual void SetMaximumHeight( int h )=0;
  
  /** 
   * Fija ancho constante
   *
   *@param w ancho del actor
   *@see bicotiActorGraphic2D::SetFixedWidth
   */
  virtual void SetFixedWidth( int w)=0;
  
  /** 
   * Fija altura constante del actor
   *
   *@param h altura del actor
   *@see bicotiActorGraphic2D::SetFixedHeight
   */
  virtual void SetFixedHeight( int h)=0;
  
  /**
   * Fija  el tamao a un valor constante
   *
   *@param w ancho del actor
   *@param h altura del actor
   *@see bicotiActorGraphic2D::SetFixedSize
   */
  virtual void SetFixedSize( int w, int h)=0;
};



#endif
