#ifndef _bicoti_actor_graphic_2D_field
#define _bicoti_actor_graphic_2D_field

#include <source/actor/actor_graphic_2D.hpp>
#include <source/interaction/interaction_field.hpp>
#include <string.h>

//---------------bicotiActorGraphic2DField---------------

/** 
 * bicotiEchoMode especifica el modo de eco de  bicotiActorGraphic2DField
 *
 * Los modos de eco posible son:
 *<pre>
 *     Modo        Descripcin
 *     ======      =================================================
 *     EchoOn      Modo normal, muestra todos los caracteres ingresados. 
 *     EchoOff     No muestra ningn caracter a medida que son ingresados
 *     Password    Muestra un asterisco en lugar de cada caracter ingresado 
 *</pre>
 */  
typedef enum { EchoOn, EchoOff, Password } bicotiEchoMode;


/**
 * Proporciona un campo para ingresar datos
 *
 * Es fuente de eventos de campo.
 */
class bicotiActorGraphic2DField : public bicotiActorGraphic2D , public bicotiInteractionSourceField
{
private:
  bool frame; 
  bicotiEchoMode echo_mode;
  int cursor_position;
  int max_length;
  static const int MAX_LENGTH = 32768;
  bool select_all;
  char * local_text;
 
protected:
  
/**
 * Verifica y sustituye el texto del campo si es distinto al del parmetro
 *
 * Este mtodo redefine el  de bicotiInteractionSourceField, para uso interno. 
 *@param new_text nuevo texto
 *@returns true si new_text es distinto del texto contenido por el campo
 */     
  bool CheckText( const char * new_text );

public:

  /**
   * Constructor
   *
   * Construye un nuevo bicotiActorGraphic2DField
   *@param  act_name nombre del actor
   */ 
  bicotiActorGraphic2DField( char * act_name = NULL );

  /** Destructor */  
  ~bicotiActorGraphic2DField();
  
  /** 
   * Fija un texto para el campo
   * 
   * El texto puede ser editado posteriormente
   *@param text el texto
   */
  void SetText( const char * text);

  /**
   * Obtiene el texto actual del campo
   *
   *@returns el texto actual del campo
   */
  const char * GetText();

  /**
   * Crea un borde alrededor del campo 
   *
   *@param set_frame si es true crea un borde, si es false el campo queda plano
   */
  void SetFrame( bool set_frame);

  /**
   * Fija el modo de eco
   *
   *@param set_echo_mode  modo de eco
   *@see bicotiEchoMode
   */
  void SetEchoMode( bicotiEchoMode set_echo_mode);
  
  /** 
   * Fija la posicin del cursor dentro del campo
   *
   *@param set_cursor nueva posicin del cursor dentro del campo
   */
  void SetCursorPosition( int set_cursor);
  
  /** 
   * Obtiene la posicin del cursor dentro del campo
   *
   *@returns la posicin del cursor dentro del campo
   */ 
  int GetCursorPosition();
  
  /** Marca todo el texto en el campo */
  void SelectAll();

  /** 
   * Indica si SelectAll ha sido invocado
   *
   *@returns true si SelectAll ha sido invocado
   *@see #SelectAll
   */
  bool AllSelected();

  /**
   * Indica si se ha fijado borde al campo
   *
   *@returns true si se ha invocado SetFrame
   *@see #SetFrame
   */
  bool GetFrame();

  /**
   * Devuelve el modo de eco fijado
   *
   *@returns el modo de eco fijado
   *@see #SetEchoMode
   */
  bicotiEchoMode GetEchoMode();

  /**
   * Obtiene la posicin inicial del cursor
   *
   *@returns la posicin inicial del cursor
   */
  int GetInitialCursorPosition();
  
  /**
   * Fija el largo mximo del texto que puede contener el campo
   *
   *@param set_max_length largo mximo del string que puede contener el campo
   */
  void SetMaxLength( int set_max_length);
  
  /**
   * Devuelve el largo mximo del texto que puede contener el campo 
   *
   *@returns largo mximo del string que puede contener el campo
   *@see #SetMaxLength
   */
  int GetMaxLength();

  /** Borra el texto del campo */
  void Clear();

};

//-----------bicotiHomologousGraphic2DField----------

/**
 * Homlogo del bicotiActorGraphic2DField
 *
 *@see bicotiActorGraphic2DField
 */
class bicotiHomologousGraphic2DField : public bicotiHomologousGraphic2D
{
public:
  
  /** Destructor */
  virtual ~bicotiHomologousGraphic2DField();
  
  /**
   * Crea un borde alrededor del campo
   *
   *@param set_frame true crea un borde
   *@see bicotiActorGraphic2DField::SetFrame
   */
  virtual void SetFrame( bool set_frame ) = 0;
  
  /**
   * Fija el modo de eco
   *  
   *@param set_echo_mode modo de eco 
   *@see bicotiActorGraphic2DField::SetEchoMode
   */ 
  virtual void SetEchoMode( bicotiEchoMode set_echo_mode) = 0;
  
  /**
   * Fija la posicin del cursor
   *
   *@param set_cursor posicin del cursor
   *@see bicotiActorGraphic2DField::SetCursorPosition
   */
  virtual void SetCursorPosition( int set_cursor ) = 0;
  
  virtual int GetCursorPosition() = 0 ;
  virtual void SetMaxLength( int ) = 0;
  virtual void SelectAll() = 0;
  void SetParameters();
  
  /** 
   * Fija un  texto para el campo 
   *
   *@param text el texto
   *@see bicotiActorGraphic2DField::SetText
   */
  virtual void SetText( const char * text)=0;
};


#endif
