#ifndef _bicoti_actor_graphic_2D_image
#define _bicoti_actor_graphic_2D_image

#include <source/actor/actor_graphic_2D.hpp>
#include <source/image_implementation/image_implementation_2D.hpp>


//------------------bicotiActorGraphic2DImage-----------------

/**@#-*/
template< class PixelType >
class bicotiImageIterator2DGraphic; 
/**@#+*/

/**
 * Actor que permite desplegar e interactuar con una imagen 2D
 *
 * Este actor tiene todas las propiedades de una imagen bicoti 2D adems de la capacidad de desplegarse en pantalla y de capturar eventos de mouse y geomtricos. Como toda imagen bicoti es template de pixel.
 *@see bicotiHomologousGraphic2DImage
 *@see bicotiImageIterator2DGraphic
 */
template< class PixelType >
class bicotiActorGraphic2DImage :  public bicotiActorGraphic2D, public bicotiImageImplementation2DSimple< PixelType >
{
  friend class bicotiImageIterator2DGraphic< PixelType >; 
   
private:
  PixelType ** ptr_pixels; 

  void DeleteMe();
  void CreateMe( PixelType, int, int, double, PixelCases );
  
  bool alpha_channel_enabled;

public:

  /**
   * Constructor
   *
   * Construye una nueva bicotiActorGraphic2DImage
   *@param def_pixel
   *@param x_0 ancho
   *@param x_1 altura
   *@param scale_value scala
   *@param pix_case
   *@param act_name nombre del actor
   */
  bicotiActorGraphic2DImage( PixelType def_pixel, int x_0 = 0, int x_1 = 0, double scale_value = 1, PixelCases pix_case = unknown, char * act_name = NULL );
  
  /**
   * Constructor de copia
   *
   * Construye una nueva bicotiActorGraphic2DImage que es copia de otra
   *@param bicotiActorGraphic2DImage<PixelType> la bicotiActorGraphic2DImage<PixelType> original.  
    */
  bicotiActorGraphic2DImage( const bicotiActorGraphic2DImage< PixelType >& imag_src ); 

  /** Destructor */ 
 ~bicotiActorGraphic2DImage();

  /**
   * Modifica un pixel
   *   
   * Modifica un pixel sobre las coordenadas indicadas. Si las coordenadas estn fuera de rango se lanza una bicotiException.
   *@param coord coordenada del pixel a modificar
   *@param value nuevo valor del pixel
   */ 
  void SetPixel( bicotiCoordinate< INTEGER >& coord, const PixelType& value );
  
  /**
   * Devuelve el valor de un pixel
   *
   * Devuelve el valor de un pixel sobre las coordenadas indicadas. Si las coordenadas estn fuera de rango se lanza una bicotiException.
   *@param coord coordenadas del pixel a devolver
   *@returns el valor del pixel
   */
  PixelType GetPixel( bicotiCoordinate< INTEGER >& coord );

  /**
   * Modifica un pixel
   *
   * Modifica un pixel sobre las coordenadas indicadas. Si las coordenadas estn fuera de rango se lanza una bicotiException.
   *@param x_0 coordenada horizontal (entero)
   *@param x_1 coordenada vertical (entero)
   *@param value nuevo valor del pixel
   */
  void SetPixel( int x_0, int x_1, const PixelType& value); 
 
  /**
   * Devuelve el valor de un pixel
   *
   * Devuelve el valor de un pixel sobre las coordenadas indicadas. Si las coordenadas estn fuera de rango se lanza una bicotiException.

   *@param x_0 coordenada horizontal (entero)
   *@param x_1 coordenada vertical (entero)
   *@returns el valor del pixel
   */  
  PixelType GetPixel( int x_0,int x_1 ); 
   
  /** 
   * Crea un iterador para la imagen
   *
   * Crea un iterador adecuado para la imagen 
   *@returns  un puntero al iterador para la imagen del mismo pixel
   */ 
  bicotiImageIterator< PixelType > * CreateIterator();

  /**
   * Crea la implemantacin de la imagen
   *
   *@returns un puntero a la implementacin de la imagen 
   */  
 bicotiImageImplementation< PixelType > * CreateImageImplementation();

  /**
   * Crea la implementacin de la imagen
   *
   * Crea la implementacin de la imagen dado un descriptor de imagen
   */ 
  bicotiImageImplementation< PixelType > * CreateImageImplementation( bicotiImageDescriptor * );

  /**
   * Actualiza la imagen en la pantalla
   *
   * RefreshImage debe invocarse despus de modificar la imagen ( por ejemplo al aplicar un algoritmo ) si se desea ver los cambios en la pantalla.
   */
  void RefreshImage();
  
  /**
   * Actualiza una porcin de la imagen en la patalla
   *
   * Actualiza una porcin rectangular de la imagen en la pantalla. Invocando este mtodo adecuadamente se puede lograr un efecto de scroll de la imagen si esta es ms grande que la ventana donde se despliega. 
   *@param x coordenada horizontal del borde superior derecho
   *@param y coordenada vertical del borde superior derecho
   *@param w ancho 
   *@param h vertical
   */
   void RefreshImage( int x, int y, int w, int h );
  
  /**
   * Habilita la transparencia
   *
   * Habilita o deshabilita la transparencia de la imagen
   *@param enable true habilita la transparecia, false la deshabilita
   */
  void EnableAlphaChannel( bool enable);

  /**
   * Indica si la transparencia est habilitada
   *
   *@returns true si la transparencia est habilitada o false si no
   *@see #EnableAlphaChannel
   */
  bool IsAlphaChannelEnabled();
};



//----------------bicotiHomologousGraphic2DImage--------------

/**
 * Homlogo de bicotiActorGraphic2DImage
 *
 *@see bicotiActorGraphic2DImage
 */
template< class PixelType >
class bicotiHomologousGraphic2DImage : public bicotiHomologousGraphic2D
{
public:

  /** Destructor */
  virtual ~bicotiHomologousGraphic2DImage(){};

  /** 
   * Pinta un pixel sobre la imagen
   *
   * Este mtodo pinta un pixel (RGBA) directamente sobre el homlogo, por lo tanto no pertenece a la imagen bicoti. Es til para dibujar unos pocos puntos sobre la imagen ya que es ms eficiente al no tener que refrescar toda la imagen.
   *@param x_in coordenada horizontal
   *@param y_in coordenada vertical 
   *@param pixel valor del pixel 
   */
  virtual void DirectSetPixel( int x_in, int y_in, bicotiPixelRGBA pixel)=0;
  
  /**
   * Borra los pixeles pintados con DirectSetPixel 
   *
   *@see #DirectSetPixel
   */
  virtual void ClearSettedPixels()=0;

  /**
   * Actualiza la imagen con los pixeles pintados con DirectSetPixel
   *
   *@see #DirectSetPixel
   */
  virtual void DirectRefresh()=0;
  
  /** 
   * Actualiza la imagen en la pantalla
   *
   *@see bicotiActorGraphic2DImage::RefreshImage
   */
  virtual void RefreshImage()=0;
  
  /**
   * Actualiza una porcin de la imagen en la pantalla
   *
   *@see bicotiActorGraphic2DImage::RefreshImage
   */
  virtual void RefreshImage( int, int, int, int )=0;
 
  /**
   * Habilita la transparencia
   *
   *@param enable true habiliata transparencia, false la deshabilita
   *@see bicotiActorGraphic2DImage::EnableAlphaChannel
   */
  virtual void EnableAlphaChannel( bool enable)=0;
  
  /**
   * Indica si la transparencia est habilitada
   *
   *@returns true si la transparencia est habilitada, false si no
   *@see bicotiActorGraphic2DImage::IsAlphaChannelEnabled
   */
  virtual bool IsAlphaChannelEnabled()=0;
};


//---------------------bicotiImageIterator2DGraphic-----------------

/**
 * Iterador para bicotiActorGraphic2DImage
 *
 *@see bicotiActorGraphic2DImage 
 */
template< class PixelType >  
class bicotiImageIterator2DGraphic : public bicotiImageIterator2D< PixelType > 
{ 
private: 
  bicotiActorGraphic2DImage< PixelType > * ptr_pixels; 
  
  void CreateMe( bicotiActorGraphic2DImage< PixelType > * ); 
  void UpdateX0(){}; 
  void Refresh(); 
  
public: 
  /** 
   * Constructor
   *
   *@param ptr_I imagen bicoti 2D
   */
  bicotiImageIterator2DGraphic( bicotiActorGraphic2DImage< PixelType >* ptr_I );
  
  /**
   * Constructor
   *
   *@param ptr_I imagen bicoti 2D
   *@param hor estrategia de borde 2D horizontal
   *@param ver estrategia de borde 2D vertical
   */
  bicotiImageIterator2DGraphic( bicotiActorGraphic2DImage< PixelType > * ptr_I, bicotiImageBorderStrategy2DX0< PixelType > hor, bicotiImageBorderStrategy2DX0< PixelType > ver);
 
  /** Destructor */
  ~bicotiImageIterator2DGraphic(){};
  /**
   * Escribe sobre el pixel actual 
   *
   *@param val
   */
  void SetCurrentPixel( PixelType val );
  
  /**
   * Fija coordenadas extremas para el iterador
   *
   * El iterador se mueve entre estas coordenadas, en un sentido ms general se puede decir que el iterador se desplaza por una subimagen de la imagen
   */
  void SetExtremeCoordinates( int first_x0, int first_x1, int last_x0, int last_x1);     
}; 
 

//========================IMPLEMENTATIONS=========================
//-----------------bicotiActorGraphic2DImage------------------
template< class PixelType >  
bicotiActorGraphic2DImage< PixelType > :: bicotiActorGraphic2DImage( PixelType def_pixel, int x_0, int x_1, double scale_value, PixelCases pix_case , char * act_name ) : bicotiActorGraphic2D( act_name )
{
  actor_case = actor_image;
  alpha_channel_enabled = false;
  CreateMe( def_pixel, x_0, x_1, scale_value, pix_case ); 
}


template< class PixelType >  
bicotiActorGraphic2DImage< PixelType > :: bicotiActorGraphic2DImage( const bicotiActorGraphic2DImage< PixelType >& imag_src )
{ 
  default_pixel = imag_src.default_pixel; 
  x0_size = imag_src.x0_size; 
  x1_size = imag_src.x1_size; 
  scale = imag_src.scale; 
  pixel_case = imag_src.pixel_case;
  alpha_channel_enabled = imag_src.alpha_channel_enabled;

  x = imag_src.x;
  y = imag_src.y;
  width = imag_src.width;
  height = imag_src.height;
  min_width = imag_src.min_width;
  max_width = imag_src.max_width;
  min_height = imag_src.min_height;
  max_height = imag_src.max_height;

  ptr_parent = imag_src.ptr_parent;

  background_color = imag_src.background_color;
  foreground_color = imag_src.foreground_color;

  is_visible = imag_src.is_visible;

  actor_case = actor_image;
  dimension_case = _2D;
  actor_type = _graphic;
  
  int i, j; 
  
  ptr_pixels = new PixelType* [ x1_size ]; 
  for ( i = 0; i < x1_size; i++ ) 
    ptr_pixels[ i ] = new PixelType[ x0_size ]; 
  for ( i =0; i < x1_size; i++ ) 
    {for ( j = 0; j < x0_size; j++ ) 
      ptr_pixels[ i ][ j ] = imag_src.ptr_pixels[ i ][ j ];}
};


template< class PixelType >  
bicotiActorGraphic2DImage< PixelType > :: ~bicotiActorGraphic2DImage()
{
  DeleteMe();
}


template< class PixelType > 
void bicotiActorGraphic2DImage< PixelType > :: DeleteMe() 
{ 
  int i; 
  
  for ( i = 0; i < x1_size; i++ ) 
    if ( x0_size > 0 ) 
      delete [] ptr_pixels[ i ]; 
  delete [] ptr_pixels;
}
 
 
template< class PixelType > 
void bicotiActorGraphic2DImage< PixelType > :: CreateMe( PixelType def_pixel, int x_0, int x_1, double scale_value, PixelCases pix_case ) 
{ 
  if ( ( x_1 >= 0 ) || ( x_0 >= 0 ) )  
    {int i, j; 
    
    default_pixel = def_pixel; 
    x0_size = x_0; 
    x1_size = x_1; 
    /*
    width = x_0;
    height = x_1;
    */
    scale = scale_value; 
    pixel_case = pix_case;
    
    ptr_pixels = new PixelType* [ x1_size ]; 
    for ( i = 0; i < x1_size; i++ ) 
      ptr_pixels[ i ] = new PixelType[ x0_size ]; 
    for ( i = 0; i < x1_size; i++ ) 
      {for ( j = 0; j < x0_size; j++ ) 
	ptr_pixels[ i ][ j ] = default_pixel;}}
  else  
    throw bicotiException( "bicotiActorGraphic2DImage", "Constructor", 2 );
}; 


template< class PixelType >  
void bicotiActorGraphic2DImage< PixelType > :: RefreshImage()
{
  if ( ptr_homologous != NULL )
    ((bicotiHomologousGraphic2DImage< PixelType >*)ptr_homologous)->RefreshImage();
}


template< class PixelType >  
void bicotiActorGraphic2DImage< PixelType > :: RefreshImage( int x, int y, int w, int h )
{
  if ( ptr_homologous != NULL )
    ((bicotiHomologousGraphic2DImage< PixelType >*)ptr_homologous)->RefreshImage( x, y, w, h );
}


template< class PixelType > 
void bicotiActorGraphic2DImage< PixelType > :: EnableAlphaChannel( bool enable )
{
  alpha_channel_enabled = enable;
  if ( ptr_homologous != NULL )
    ((bicotiHomologousGraphic2DImage< PixelType >*)ptr_homologous)->EnableAlphaChannel( enable );

}


template< class PixelType >  
bool bicotiActorGraphic2DImage< PixelType > :: IsAlphaChannelEnabled()
{
  return alpha_channel_enabled;
}


template< class PixelType >  
void bicotiActorGraphic2DImage< PixelType > :: SetPixel( bicotiCoordinate< INTEGER >& coord, const PixelType& value )
{ 
  unsigned x_0, x_1;
  bicotiCoordinate2D< INTEGER > aux_coord = (bicotiCoordinate2D< INTEGER >) coord;
  
  x_0 = aux_coord.X0();
  x_1 = aux_coord.X1();
  
#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( (  x_1 + 1 ) > x1_size ) || ( x_0 < 0 ) || ( ( x_0 + 1 ) > x0_size ) ) 
    throw bicotiException( "bicotiActorGraphic2DImage", "SetPixel()", 1 );
#endif
  
  ptr_pixels[ x_1 ][ x_0 ] = value; 
}; 


template< class PixelType >  
PixelType bicotiActorGraphic2DImage< PixelType > :: GetPixel( bicotiCoordinate< INTEGER >& coord ) 
{ 
  unsigned x_0, x_1;
  bicotiCoordinate2D< INTEGER > aux_coord = (bicotiCoordinate2D< INTEGER >) coord;
  
  x_0 = aux_coord.X0();
  x_1 = aux_coord.X1();
  
#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( (  x_1 + 1 ) > x1_size ) || ( x_0 < 0 ) || ( ( x_0 + 1 ) > x0_size ) ) 
    throw bicotiException( "bicotiActorGraphic2DImage", "GetPixel()", 1 );
#endif
  
  return ptr_pixels[ x_1 ][ x_0 ]; 
}; 

 
template< class PixelType >  
void bicotiActorGraphic2DImage< PixelType > :: SetPixel( int x_0, int x_1, const PixelType& value )
{ 
#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( (  x_1 + 1 ) > x1_size ) || ( x_0 < 0 ) || ( ( x_0 + 1 ) > x0_size ) ) 
    throw bicotiException( "bicotiActorGraphic2DImage", "SetPixel()", 1 );
#endif

  ptr_pixels[ x_1 ][ x_0 ] = value; 
}; 


template< class PixelType >  
PixelType bicotiActorGraphic2DImage< PixelType > :: GetPixel( int x_0, int x_1 )
{ 
#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( (  x_1 + 1 ) > x1_size ) || ( x_0 < 0 ) || ( ( x_0 + 1 ) > x0_size ) ) 
    throw bicotiException( "bicotiActorGraphic2DImage", "GetPixel()", 1 );
#endif

  return ptr_pixels[ x_1 ][ x_0 ]; 
}; 
 
 
template< class PixelType >  
bicotiImageIterator< PixelType > * bicotiActorGraphic2DImage< PixelType > :: CreateIterator() 
{ 
  return( new bicotiImageIterator2DGraphic< PixelType >( this ) );
}; 


template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiActorGraphic2DImage< PixelType > :: CreateImageImplementation()
{
  return( new bicotiActorGraphic2DImage< PixelType >( default_pixel, x0_size, x1_size, scale, pixel_case ) );
};


template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiActorGraphic2DImage< PixelType > :: CreateImageImplementation( bicotiImageDescriptor * ptr_desc )
{
  double scale_aux = ptr_desc->GetScale();
  PixelCases pix_case_aux = ptr_desc->GetPixelCase();
  
  if ( ( ptr_desc->GetDimension() ) != 2 )
    throw bicotiException( "bicotiActorGraphic2DImage", "CreateImageImplementation()", 2 );
  
  unsigned x0_size_aux = ptr_desc->Size( 0 );
  unsigned x1_size_aux = ptr_desc->Size( 1 );
  
  return( new bicotiActorGraphic2DImage< PixelType >( default_pixel, x0_size_aux, x1_size_aux, scale_aux, pix_case_aux ) );
};


//------------------bicotiImageIterator2DGraphic--------------------
template< class PixelType >  
void bicotiImageIterator2DGraphic< PixelType > :: CreateMe( bicotiActorGraphic2DImage< PixelType > * ptr_I ) 
{ 
  if ( (ptr_I->GetX0Size() != 0 ) && ( ptr_I->GetX1Size() != 0 ) ) 
    {is_done = false; 
    ptr_pixels = ptr_I; 
    first_x1 = 0; 
    first_x0 = 0; 
    last_x1 = ptr_pixels->x1_size - 1; 
    last_x0 = ptr_pixels->x0_size - 1; 
    current_x0 = first_x0; 
    current_x1 = first_x1; 
    Refresh();} 
  else 
    throw bicotiException( "bicotiActorGraphic2DImage", "CreateMe()", 6 );
}; 
   
     
template< class PixelType >  
bicotiImageIterator2DGraphic< PixelType > :: bicotiImageIterator2DGraphic( bicotiActorGraphic2DImage< PixelType > * ptr_I ) 
{   
  CreateMe( ptr_I ); 
  x0_strategy = bicotiImageBorderStrategy2DX0< PixelType >( this ); 
  x1_strategy = bicotiImageBorderStrategy2DX1< PixelType >( this );  
}; 

 
template< class PixelType >  
bicotiImageIterator2DGraphic< PixelType > :: bicotiImageIterator2DGraphic( bicotiActorGraphic2DImage< PixelType >* ptr_I, bicotiImageBorderStrategy2DX0< PixelType > hor, bicotiImageBorderStrategy2DX0< PixelType > ver ) 
{   
  CreateMe( ptr_I ); 
  x0_strategy = hor; 
  x1_strategy = ver; 
  x0_strategy.UseIterator( this ); 
  x1_strategy.UseIterator( this ); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2DGraphic< PixelType > :: SetCurrentPixel( PixelType val ) 
{  
  *ptr_current = val;  
}; 


template< class PixelType >  
inline void bicotiImageIterator2DGraphic< PixelType > :: Refresh() 
{   
  ptr_current = ptr_pixels->ptr_pixels[ current_x1 ] + current_x0;  
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2DGraphic< PixelType > :: SetExtremeCoordinates( int first_x_0, int first_x_1, int last_x_0, int last_x_1 ) 
{ 
  if( ( last_x_1 < first_x_1 ) || ( last_x_0 < first_x_0 ) ) 
    throw bicotiException( "bicotiImageIterator2DGraphic", "SetExtremeCoordinates()", 1 );
  else 
    {if( ( last_x_1 >= 0 ) && ( last_x_1 < ptr_pixels->GetX1Size() ) ) 
      last_x1 = last_x_1; 
    else 
      throw bicotiException( "bicotiImageIterator2DGraphic", "SetExtremeCoordinates()", 1 );};
  
  if( ( last_x_0 >= 0 ) && ( last_x_0 < ptr_pixels->GetX0Size() ) ) 
    last_x0 = last_x_0; 
  else 
    throw bicotiException( "bicotiImageIterator2DGraphic", "SetExtremeCoordinates()", 1 );
  
  first_x1 = first_x_1; 
  first_x0 = first_x_0; 
  First();
};                  


#endif
