#ifndef _bicoti_interaction_field
#define _bicoti_interaction_field

#include <stl.h>
#include <source/actor/actor_basic.hpp>

//--------------------DECLARATIONS---------------------
//-----------------bicotiListenerField----------------

/** 
 * Listener de eventos de campo
 *
  * Extienda esta clase en su aplicacin implementando los mtodos de campo para que hagan lo que usted desea.
 *Ejemplo:
 *<pre>
 *@@  class MyListenerField : public bicotiListenerField;
 *@@  { void FieldReturnPressed( bicotiActor * ptr_actor );
 *@@    void FieldTextChanged( bicotiActor * ptr_actor ); }
 *</pre>
 *@see bicotiInteractionSourceField  
 */ 
class bicotiListenerField
{
public:

  /**
   * Ejecutada cuando se oprime enter dentro de un campo
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerField :: FieldReturnPressed( bicotiActor * ptr_actor )
   *@@  { cout << " Oprimieron enter " << endl; }
   *<pre>
   *@param ptr_actor puntero al actor que produjo el evento
   */
  virtual void FieldReturnPressed( bicotiActor * )=0;
  
 /**
   * Ejecutada cuando se cambia el texto del campo
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerField :: FieldTextChanged( bicotiActor * ptr_actor )
   *@@  { cout << " Modificando el texto " << endl; }
   *<pre>
   *@param ptr_actor puntero al actor que produjo el evento
   */
  virtual void FieldTextChanged( bicotiActor * ptr_actor)=0;
};



//-----------bicotiInteractionSourceField-----

/** 
 * Fuente de eventos de campo
 *
 * Clase base para bicotiActionGraphic2DField
 *@see bicotiActorGraphic2DField
 */
class bicotiInteractionSourceField
{
protected:
  vector< bicotiListenerField * > listener_text;

  /** 
   * Cambia el texto del actor campo si se produjo algun cambio
   *
   * Implementada en bicotiActorGraphic2DField
   *@param text texto nuevo
   *@returns true si text difiere del contenido por el actor
   */
  virtual bool CheckText( const char * text)=0;
  
public:
  
  /**
   * Registra un listener de eventos de campo
   *
   *@param ptr_listener listener para agregar
   */
  void AddListenerField( bicotiListenerField * ptr_listener);
  
  /**
   * Quita un listener de eventos de campo
   *
   *@param ptr_listener listener para quitar
   */
  void RemoveListenerField( bicotiListenerField * );
  
  /**
   * Dispara eventos de texto de capo cambiado
   *
   * Avisa a todos los listeners registrados que ha ocurrido un evento de texto modificado
   *@param ptr_actor actor que dispara el evento
   *@param new_text nuevo texto
   */
  void FireFieldChanged( bicotiActor * ptr_actor, const char * new_text );
  
   /**
   * Dispara eventos de tecla return oprimida 
   *
   * Avisa a todos los listeners registrados que ha ocurrido un evento de return oprimido
   *@param ptr_actor actor que dispara el evento
   */
  void FireReturnPressed( bicotiActor * );

};

#endif

