#ifndef _bicoti_interaction_file_dialog
#define _bicoti_interaction_file_dialog

#include <stl.h>
#include <source/actor/actor_basic.hpp>

//--------------------DECLARATIONS---------------------
//-----------------bicotiListenerFileDialog----------------

/** 
 * Listener de eventos de un dilogo de archivo
 *
 * clase base para bicotiActorGraphic2DFileDialog
 *@see bicotiActorGraphic2DFileDialog
 */
class bicotiListenerFileDialog
{
public:

  /** 
   * Este mtodo se ejecuta cuando se selecciona un archivo con doble click o boton OK/SAVE en un dilogo de archivo
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerFileDialog :: FileDialogFileSelected( bicotiActor * ptr_actor, const char * file_name )
   *@@  { bicotiDeviceFile device;
   *@@    device->Open("rw");
   *@@    ...process file ...
   *@@    device->Close(); }
   *</pre>
   *@param ptr_actor puntero al actor que produjo el evento 
   *@param file_name nombre del arcivo seleccionado
   */ 
  virtual void FileDialogFileSelected( bicotiActor * ptr_actor, const char * file_name)=0;
  
  /** 
   * Este mtodo se ejecuta cuando se destaca un archivo en un dialogo de archivo
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerFileDialog :: FileDialogFileHighlighted( bicotiActor * ptr_actor, const char * file )
   *@@  { cout << " Selecion el archivo: " << file << endl; }
   *</pre>
   *@param ptr_actor puntero al actor que produjo el evento 
   *@param file nombre del arcivo destacado
   */ 
  virtual void FileDialogFileHighlighted( bicotiActor * ptr_actor, const char * file )=0;

/** 
 * Este mtodo se ejecuta cuando se cambia de carpeta en un dialogo de archivo
 *
 * Debe reescribir este mtodo en su aplicacin
 * Ejemplo:
 *<pre>
 *@@  void MyListenerFileDialog :: FileDialogFileHighlighted( bicotiActor * ptr_actor, const char * file )
 *@@  { cout << " Entrando a la carpeta: " << directory << endl; }
 *</pre>
 *@param ptr_actor puntero al actor que produjo el evento 
 *@param directory nombre de la carpeta a la que se movi
 */
  virtual void FileDialogDirectoryChanged( bicotiActor * ptr_actor, const char * newdir )=0;

};



//-----------bicotiInteractionSourceFileDialog-----

/**
 * Fuente de eventos de dilogo de archivo
 *
 * clase base para bicotiActorGraphic2DFileDialog
 *@see bicotiActorGraphic2DField
 */
class bicotiInteractionSourceFileDialog
{
protected:
  vector< bicotiListenerFileDialog * > listener_file_dialog;
  
  /**
   * Verifica y sustituye el directorio actual con el del parmetro si es distinto
   *@param newdir nombre del nuevo directorio 
   */
  virtual bool CheckCurrentDirectory( const char * newdir )=0;

public:

  /**
   * Registra un listener de eventos de dilogo de archivos
   *
   *@param ptr_listener listener de eventos a agregar
   */
  void AddListenerFileDialog( bicotiListenerFileDialog * ptr_listener);
 
  /**
   * Quita un listener de eventos de dilogo de archivo
   *
   *@param ptr_listener listener de eventos a quitar
   */
  void RemoveListenerFileDialog( bicotiListenerFileDialog * ptr_listener);
  
  /**
   * Dispara eventos de archivo seleccionado
   *
   * Este mtodo avisa a todo los listener registrados que se ha seleccionado un archivo
   *@param ptr_act actor que dispara el evento
   *@param file nombre del archivo seleccionado  
   */ 
  void FireFileDialogFileSelected( bicotiActor * ptr_act, const char * file);
  
  /**
   * Dispara eventos de archivo destacado
   *
   * Este mtodo avisa a todo los listener registrados que se ha destacado un archivo (highlighted)
   *@param ptr_act actor que dispara el evento
   *@param file nombre del archivo destacado
   */ 
  void FireFileDialogFileHighlighted( bicotiActor * ptr_act, const char * file);
  
  /**
   * Dispara eventos de directorio cambiado
   *
   * Este mtodo avisa a todo los listener registrados que se ha cambiado de directorio
   *@param ptr_act actor que dispara el evento
   *@param file nombre del archivo destacado
   */ 
  void FireFileDialogDirectoryChanged( bicotiActor * ptr_act, const char * newdir );
};



#endif

