#ifndef _bicoti_interaction_geometry
#define _bicoti_interaction_geometry

#include <stl.h>
#include <source/actor/actor_basic.hpp>

//--------------------DECLARATIONS---------------------
//-----------------bicotiEventGeometry-----------------

/**
 * Evento de geometra
 *
 * encapsula datos de inters para este tipo de evento
 *Se entiende eventos de geometra al cambio de tamao y posicin de los actores.
 */
class bicotiEventGeometry
{
private:
  int x, y;

public:
  
  /**
   * Constructor
   * 
   * Construye un nuevo evento a partir de los datos de posicin o tamao del actor dependiendo del tipo de evento.
   *@param x_in coordenada horizontal o ancho del actor
   *@param y_in coordenada vertical o altura del actor
   */
  bicotiEventGeometry( int x_in, int y_in);
  
  /**
   * Constructor por defecto
   * 
   * Construye un nuevo evento, inicializando los valores en 0.
   */
  bicotiEventGeometry();
  
  /**
   * Devuelve la coordenada horizonta o ancho del actor
   *
   *@returns coordenada horizontal o ancho del actor 
   */
  int GetX() const;
 
  /**
   * Devuelve la coordenada vertical o ancho del actor
   *
   *@returns coordenada vertical o ancho del actor 
   */
  int GetY() const;
};


//----------------bicotiListenerGeometry---------------

/** 
 * Listener de eventos de geometra
 *
 * Se entiende eventos de geometra al cambio de tamao y posicin de los actores.
 * Extienda esta clase en su aplicacin implementando los mtodos de la misma para que haga lo que usted desea.
 *Ejemplo:
 *<pre>
 *@@  class MyListenerMouseGeometry : public bicotiListenerGeometry;
 *@@  { void ActorResized( bicotiActor * ptr_actor, const bicotiEventGeometry& event ); 
 *@@    void ActorMoved( bicotiActor * ptr_actor, const bicotiEventGeometry& event ); }
 *</pre>
 *@see bicotiInteractionSourceGeometry  
 */ 
class bicotiListenerGeometry
{
public:
 /**
   * Este mtodo se ejecuta cuando el actor cambia de tamao
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerGeometry :: ActorResized( bicotiActor * ptr_actor, const bicotiEventGeometry& event )
   *@@  { cout << " Ancho del actor: " << event->GetX() << " Altura del actor: " << event->GetY << endl; }
   *</pre>
   *@param ptr_actor puntero al actor que produjo el evento
   *@param event datos del evento 
   */
  virtual void ActorResized( bicotiActor * ptr_actor, const bicotiEventGeometry& event )=0;

 /**
   * Este mtodo se ejecuta cuando el actor es movido en la pantalla
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerGeometry :: ActorMoved( bicotiActor * ptr_actor, const bicotiEventGeometry& event )
   *@@  { cout << " Nueva posicin: x = " << event->GetX() << " y = " << event->GetY << endl; }
   *</pre>
   *@param ptr_actor puntero al actor que produjo el evento
   *@param event datos del evento 
   */
  virtual void ActorMoved( bicotiActor * ptr_actor, const bicotiEventGeometry&  event )=0;
};


//--------------bicotiInteractionSourceGeometry--------

/**
 * Fuente de eventos de geometra
 *
 * Se entiende eventos de geometra al cambio de tamao y posicin de los actores.
 * Clase base para bicotiActorGraphic2D
 *@see bicotiActorGraphic2D
 */
class bicotiInteractionSourceGeometry
{
protected:
  vector< bicotiListenerGeometry * > listener_container;

public:

  /**
   * Registra un listener de eventos de geometra
   *
   *@param ptr_listener listener de eventos a registrar
   */
  void AddListenerGeometry( bicotiListenerGeometry * ptr_listener);
  
/**
   * Quita un listener de eventos de geometra
   *
   *@param ptr_listener listener de eventos a quitar
   */ 
void RemoveListenerGeometry( bicotiListenerGeometry * );

 /** 
   * Dispara eventos de geometra cuando cambia el tamao del actor
   *
   * Este mtodo avisa a todo los listener registrados que el actor ha cambiado de tamao 
   *@param ptr_act actor que dispara el evento
   *@param x_in ancho del actor
   *@param y_in altura del actor
   */
  void FireActorResized( bicotiActor * ptr_actor, int x_in, int y_in);

/** 
   * Dispara eventos de geometra cuando cambia la posicin del actor
   *
   * Este mtodo avisa a todo los listener registrados que se ha movido el actor en la pantalla 
   *@param ptr_act actor que dispara el evento
   *@param x_in coordenada horizonta del actor 
   *@param y_in coordenada vertical del actor
   */
  void FireActorMoved( bicotiActor * ptr_actor, int x_in, int y_in);

  /**
   * Verifica si cambi la posicin del actor 
   *
   * Verifica si cambi la posicin del actor y actualiza al actor con la nueva posicin. 
   *@param x_in coordenada horizontal de la nueva posicin
   *@param y_in coordenada vertical de la nueva posicin
   */
  virtual bool CheckSize( int x_in, int y_in)=0;
 
   /**
   * Verifica si cambi la posicin del actor 
   *
   * Verifica si cambi la posicin del actor y actualiza al actor con la nueva posicin.
   *@param x_in coordenada horizontal de la nueva posicin
   *@param y_in coordenada vertical de la nueva posicin
   */
  virtual bool CheckPosition( int x_in, int y_in)=0;
};


#endif
