#ifndef _bicoti_interaction_mouse
#define _bicoti_interaction_mouse

#include <stl.h>
#include <source/actor/actor_basic.hpp>

//--------------------DECLARATIONS---------------------

/**
 * Identificacin de los botones del ratn
 *
 * estos valores identifican respectivamente botn izquierdo, botn del medio y botn derecho. Pueden combinarse entre si
 */
enum button_cases { button_left=1, button_middle=2, button_right=4  };
/** 
 * Tipo de los indentificadores de botn
 *
 *@see button_cases
 */
typedef enum button_cases MouseButtonCases;


//------------------bicotiEventMouse-------------------

/**
 * Evento de mouse
 *
 * Encapsula datos de inters para este tipo de eventos
 */
class bicotiEventMouse
{
private:
  int x, y;
  char button_mask;

public:

  /**
   * Constructor
   *
   * Costruye un nuevo evento a partir de los datos de posicin de mouse ( cuando se precion o liber el botn ) y la identificacin de los botones precionados.
   *@param x_in coordenada horizontal del ratn
   *@param y_in coordenada vertical del ratn
   *@param mask mscara que identifica los botones del ratn accionados
   */
  bicotiEventMouse( int x_in, int y_in, char mask);
  
  /**
   * Devuelve la coordenada horizontal del mouse
   *
   *@returns coordenada horizontal del mouse al accionar sus botones
   */
  int GetX() const;
   
  /**
   * Devuelve la coordenada vertical del mouse
   *
   *@returns coordenada vertical del mouse al accionar sus botones
   */
  int GetY() const;

  /**
   * Devuelve cual o cuales botones fueron precionados
   *
   *@returns un valor que identifica el o los botones precionados (o liberados)
   */ 
  char GetButtonMask() const;
  
  /**
   * Indica si se precion el botn izquierdo
   *
   *@returns true si uno de los botones precionados fue el izquierdo
   */
  bool IsLeftButton() const;
  
  /**
   * Indica si se precion el botn del medio
   *
   *@returns true si uno de los botones precionados fue el del medio
   */
  bool IsMiddleButton() const;
  
  /**
   * Indica si se precion el botn derecho
   *
   *@returns true si uno de los botones precionados fue el derecho
   */
  bool IsRightButton() const;
};



//------------------bicotiListenerMouse----------------

/** 
 * Listener de eventos de de ratn
 *
 * Extienda esta clase en su aplicacin implementando los mtodos  que hagan lo que usted desea.
 *Ejemplo:
 *<pre>
 *@@  class MyListenerMouse : public bicotiListenerMouse;
 *@@  { void MousePressed( bicotiActor * ptr_actor, const bicotiEventMouse& event );
 *@@    void MouseRelease( bicotiActor * ptr_actor, const bicotiEventMouse& evet }
 *</pre>
 *@see bicotiInteractionSourceMouse 
 */ 
class bicotiListenerMouse
{
public:

  /**
   * Este mtodo se ejecuta cuando se presiona algun botn del ratn
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerMouse :: MousePressed( bicotiActor * ptr_actor, const bicotiEventMouse& event )
   *@@  { cout << " La posicin del ratn es: " << event->GetX() << " , " << event->GetY << endl; }
   *</pre>
   *@param ptr_actor puntero al actor que produjo el evento
   *@param event datos del evento 
   */
  virtual void MousePressed( bicotiActor * ptr_actor, const bicotiEventMouse&  event )=0;

  /**
   * Este mtodo se ejecuta cuando se libera algun botn del ratn
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerMouse :: MouseReleased( bicotiActor * ptr_actor, const bicotiEventMouse& event )
   *@@  { cout << " La posicin del ratn es: " << event->GetX() << " , " << event->GetY << endl; }
   *</pre>
   *@param ptr_actor puntero al actor que produjo el evento
   *@param event datos del evento 
   */
  virtual void MouseReleased( bicotiActor * ptr_actor, const bicotiEventMouse& event )=0;
};



//-------------bicotiInteractionSourceMouse------------

/** 
 * Fuente de eventos de ratn
 *
 * clase base para bicotiActorGraphic
 *@see bicotiActorGraphic
 */ 
class bicotiInteractionSourceMouse
{
protected:
  vector< bicotiListenerMouse * > listener_mouse;
  
public:

  /**
   * Registra un listener de eventos de ratn
   *
   *@param ptr_listener listener de eventos a registrar
   */
  void AddListenerMouse( bicotiListenerMouse * ptr_listener );
  
  /**
   * Quita un listener de eventos de ratn
   *
   *@param ptr_listener listener de eventos a quitar
   */ 
  void RemoveListenerMouse( bicotiListenerMouse * ptr_listener );

  
  /** 
   * Dispara un evento de ratn cuando se preciona un botn del mismo
   *
   * Este mtodo avisa a todo los listener registrados que se ha movido el mouse 
   *@param ptr_act actor que dispara el evento
   *@param x_in coordenada horizonta del ratn
   *@param y_in coordenada vertical del ratn
   *@param mask mascara que identifica los botones del ratn precionados
   */
  void FireMousePressed( bicotiActor * ptr_act, int x_in, int y_in, char mask);

  /** 
   * Dispara un evento de ratn cuando se preciona un botn del mismo
   *
   * Este mtodo avisa a todo los listener registrados que se ha movido el mouse 
   *@param ptr_act actor que dispara el evento
   *@param x_in coordenada horizonta del ratn
   *@param y_in coordenada vertical del ratn
   *@param mask mascara que identifica los botones del ratn liberados
   */
  void FireMouseReleased( bicotiActor * ptr_act, int x_in, int x_in, char mask);
};


#endif

