#ifndef _bicoti_interaction_mouse_motion
#define _bicoti_interaction_mouse_motion

#include <stl.h>
#include <source/actor/actor_basic.hpp>


//---------------bicotiEventMouseMotion----------------

/**
 * Evento de movimiento de mouse
 *
 * encapsula datos de inters para este tipo de evento
 */
class bicotiEventMouseMotion
{
private:
  int x, y;

public:
  
  /**
   * Constructor
   * 
   * Construye un nuevo evento a partir de los datos de posicin del mouse
   *@param x_in coordenada horizontal del mouse
   *@param y_in coordenada vertical del mouse
   */
  bicotiEventMouseMotion( int x_in, int y_in);
  
  /**
   * Devuelve la coordenada vertical del mouse
   *
   *@returns coordenada horizontal del mouse
   */
  int GetX() const;
  
  /**
   * Devuelve la coordenada vertical del mouse
   *
   *@returns coordenada vertical del mouse
   */
  int GetY() const;
};



//--------------bicotiListenerMouseMotion-------------

/** 
 * Listener de eventos de movimiento de ratn
 *
 * Extienda esta clase en su aplicacin implementando el mtodo MouseMoved para que haga lo que usted desea.
 *Ejemplo:
 *<pre>
 *@@  class MyListenerMouseMotion : public bicotiListenerMouseMotion;
 *@@  { void MouseMoved( bicotiActor * ptr_actor, const bicotiEventMouseMotion& event ); }
 *</pre>
 *@see bicotiInteractionSourceMouseMotion  
 */ 
class bicotiListenerMouseMotion
{
public:

  /**
   * Este mtodo se ejecuta cuando se mueve el ratn
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerMouseMotion :: MouseMoved( bicotiActor * ptr_actor, const bicotiEventMouseMotion& event )
   *@@  { cout << " La posicin del ratn es: " << event->GetX() << " , " << event->GetY << endl; }
   *</pre>
   *@param ptr_actor puntero al actor que produjo el evento
   *@param event datos del evento 
   */
  virtual void MouseMoved( bicotiActor * ptr_actor, const bicotiEventMouseMotion& event )=0;
};



//----------bicotiInteractionSourceMouseMotion---------

/** 
 * Fuente de eventos de movimiento de ratn
 *
 * clase base para bicotiActorGraphic
 *@see bicotiActorGraphic
 */
class bicotiInteractionSourceMouseMotion
{
protected:
  vector< bicotiListenerMouseMotion * > listener_mouse_motion;
  
  /** 
   * Activa los eventos de movimiento de ratn
   *
   *@see bicotiActorGraphic
   */
  virtual void ActivateMouseMotionTracking()=0;

  /**
   * Desactiva eventos de movimiento de ratn
   *
   *@see bicotiActorGraphic
   */
  virtual void DeactivateMouseMotionTracking()=0;
  
public:

  /**
   * Registra un listener de eventos de movimiento de ratn
   *
   *Al agregar el primer listener se activan los eventos de movimiento de mouse. Esto garantiza que no se atendern esos eventos a menos que halla alguien interesado en ellos.  
   *@param ptr_listener listener de eventos a registrar
   */
  void AddListenerMouseMotion( bicotiListenerMouseMotion * ptr_listener );
  
  /**
   * Quita un listener de eventos de movimiento de ratn
   *
   * Al quitar el ltimo listener se desactivan los eventos de movimiento de mouse.
   *@param ptr_listener listener de eventos a quitar
   */ 
  void RemoveListenerMouseMotion( bicotiListenerMouseMotion * ptr_listener );

  /** 
   * Dispara eventos de movimiento de ratn
   *
   * Este mtodo avisa a todo los listener registrados que se ha movido el mouse 
   *@param ptr_act actor que dispara el evento
   *@param x_in coordenada horizonta del ratn
   *@param y_in coordenada vertical del ratn
   */
  void FireMouseMoved( bicotiActor * ptr_act, int x_in, int y_in );
};


#endif

