#ifndef _bicoti_interaction_scroll_bar
#define _bicoti_interaction_scroll_bar


#include <stl.h>
#include <source/actor/actor_basic.hpp>


//--------------------DECLARATIONS---------------------
//-----------------bicotiEventScrollBar-----------------

/**
 * Evento de rango del scrollbar
 *
 * encapsula datos de inters para este tipo de evento
 */
class bicotiEventScrollBarRange
{
private:
  int min, max;

public:

  /**
   * Constructor
   * 
   * Construye un nuevo evento a partir de los datos de rango
   *@param minimum extremo inferior del rango
   *@param maximum extremo superior del rango
   */
  bicotiEventScrollBarRange( int minimum, int maximum);
  
  /**
   * Devuelve el mnimo del rango
   * 
   *@returns extremo inferior del rango
   */
  int GetMinimum() const;

  /**
   * Devuelve el mximo del rango
   * 
   *@returns extremo superior del rango
   */
  int GetMaximum() const;
};


/** 
 * Evento de cambio del paso del scroll bar
 *
 * encapsula datos de inters para este tipo de evento
 */
class bicotiEventScrollBarStep
{
private:
  int line_step, page_step;

public:

  /**
   * Constructor
   *
   * construye un nuevo evento a partir de los pasos del scroll bar
   *@params ls paso de lnea
   *@params ps paso de pgina
   */ 
  bicotiEventScrollBarStep( int ls, int ps);
  
  /**
   * Devuelve paso de lnea
   *
   *@returns paso de lnea
   */
  int GetLineStep() const;
  /**
   * Devuelve paso de pgina
   *
   *@returns paso de pgina
   */
  int GetPageStep() const;
};


//-----------------bicotiListenerScrollBar----------------

/** 
 * Listener de eventos de scrollbar
 *
 * Extienda esta clase en su aplicacin implementando los mtodos de scroll bar para que hagan lo que usted desea.
 *Ejemplo:
 *<pre>
 *@@  class MyListenerScrollBar : public bicotiListenerScrollBar;
 *@@  { void ScrollBarValueChanged( bicotiActor * ptr_actor, int val );
 *@@    void ScrollBarRangeChanged( bicotiActor * ptr_actor, const bicotiEventScrollBarRange & event);
 *@@    void ScrollBarStepChanged( bicotiActor * ptr_actor, const bicotiEventScrollBarStep & event);
 *@@    void ScrollBarSliderPressed( bicotiActor * ptr_actor, int val );
 *@@    void ScrollBarSliderReleased( bicotiActor * ptr_actor, int val ); 
 *@@  }
 *</pre>
 *@see bicotiInteractionSourceScrollBar  
 */ 
class bicotiListenerScrollBar
{
public:
  virtual void ScrollBarValueChanged( bicotiActor *, int )=0;
  virtual void ScrollBarRangeChanged( bicotiActor *, const bicotiEventScrollBarRange & )=0;
  virtual void ScrollBarStepChanged( bicotiActor *, const bicotiEventScrollBarStep & )=0;
  virtual void ScrollBarSliderPressed( bicotiActor *, int )=0;
  virtual void ScrollBarSliderReleased( bicotiActor *, int )=0;
};



//-----------bicotiInteractionSourceScrollBar-----

/** 
 * Fuente de eventos de scroll bar
 *
 * clase base para bicotiActorGraphic2DScrollBar
 *@see bicotiActorGraphic2DScrollBar
 */
class bicotiInteractionSourceScrollBar
{
protected:
  vector< bicotiListenerScrollBar * > listener_scroll_bar;
  
public:
 
 /**
   * Registra un listener de eventos de scroll bar
   *
   *@param ptr_listener listener de eventos a agregar
   */
  void AddListenerScrollBar( bicotiListenerScrollBar * ptr_listener);
 
   /**
   * Quita un listener de eventos de scroll bar
   *
   *@param ptr_listener listener de eventos a quitar
   */
  void RemoveListenerScrollBar( bicotiListenerScrollBar * ptr_listener);

  /**
   * Dispara eventos de cambio de valor de la scroll bar
   *
   * Este mtodo avisa a todo los listener registrados que ha ocurrido un evento de cambio de valor (deslizamiento)
   *@param ptr_actor actor que dispara el evento
   *@param val valor del scrollbar al producirse el evento
   */ 
  void FireScrollBarValueChanged( bicotiActor * ptr_actor, int val );
  
    /**
   * Dispara eventos de cambio de rango de la scroll bar
   *
   * Este mtodo avisa a todo los listener registrados que ha ocurrido un evento de cambio de rango 
   *@param ptr_actor actor que dispara el evento
   *@param min mnimo del rango
   *@param max mximo del rango
   */ 
  void FireScrollBarRangeChanged( bicotiActor * ptr_actor, int min, int max);
  
    /**
   * Dispara eventos de cambio de paso de la scroll bar
   *
   * Este mtodo avisa a todo los listener registrados que ha ocurrido un evento de cambio de paso
   *@param ptr_actor actor que dispara el evento
   *@param line paso de linea
   *@param page paso de pgina
   */ 
  void FireScrollBarStepChanged( bicotiActor * ptr_actor, int line, int page);
  
  /**
   * Dispara eventos de slider oprimido
   *
   * Este mtodo avisa a todo los listener registrados que ha ocurrido un evento de slider oprimido
   *@param ptr_actor actor que dispara el evento
   *@param val valor del scrollbar al producirse el evento
   */ 
  void FireScrollBarSliderPressed( bicotiActor *ptr_actor, int  val );
  
   /**
   * Dispara eventos de slider liberado
   *
   * Este mtodo avisa a todo los listener registrados que ha ocurrido un evento de slider liberado
   *@param ptr_actor actor que dispara el evento
   *@param val valor del scrollbar al producirse el evento
   */
  void FireScrollBarSliderReleased( bicotiActor * ptr_actor, int val );

  /** 
   * Actualiza el estado del actor si el valor es distinto 
   *
   * Implementado en bicotiActorGraphic2DScrollBar
   *@param val nuevo valor del scrollbar
   *@returns true si val es distinto del valor del actor
   */
  virtual bool CheckScrollBarValue( int val )=0;
  
  /** 
   * Actualiza el estado del actor si el valor es distinto 
   *
   * Implementado en bicotiActorGraphic2DScrollBar
   *@param min mnimo del rango
   *@param max mximo del rango
   *@returns true si min o max son distintos difiere de los del actor 
   */
  virtual bool CheckScrollBarRange( int min, int max)=0;
  
   /** 
   * Actualiza el estado del actor si el valor es distinto 
   *
   * Implementado en bicotiActorGraphic2DScrollBar
   *@param ls paso de linea
   *@param ps paso de pgina
   *@returns true si ls o ps difieren de los del actor
   */
  virtual bool CheckScrollBarStep( int ls , int ps )=0;
};



#endif

