#ifndef _mapper_jpg
#define _mapper_jpg


#include <source/mapper/mapper_basic.hpp>


//------------------bicotiMapperJPEG-------------------

/** 
 * Espacios de color
 *
 *<pre>
 *   Tipo            Descripcin
 * ==========        =====================
 * cs_unknown        espacio de color no definido
 * cs_grayscale      escala de grises
 * cs_rgb            RGB
 * cs_YCbCr          Y/Cb/Cr (YUV)          
 * cs_CMYK           C/M/Y/K 
 * cs_YCCK           Y/Cb/Cr/K
 *</pre>
 */  
typedef enum { cs_unknown, cs_grayscale, cs_rgb, cs_YCbCr, cs_CMYK, cs_YCCK } bicotiColorSpace; 

/**
 * Tratamientos de dither
 *
 *<pre>
 * Tipo                       Descripcin
 * ==================         ==============
 * dm_none                    ninguno
 * dm_ordered                 ordenado
 * dm_floyd_steinberg         floyd-steinberg   
 *</pre>
 */
typedef enum { dm_none, dm_ordered, dm_floyd_steinberg} bicotiDitherMode;

/**
 * Clase base para los Mappers JPEG
 *
 *@see bicotiMapperJPEGIJG
 */
template< class PixelType >
class bicotiMapperJPEG : public bicotiMapper< PixelType >
{
protected:
  bicotiColorSpace file_colorspace;
  bicotiColorSpace image_colorspace;
  bicotiDitherMode dither_mode ; //JDITHER_NONE, JDITHER_ORDERED, JDITHER_FS
  int quality;
  int scale_num, scale_den;
  bool force_baseline;
  bool csFlag; //color_space is set
  bool qtFlag; //quality is set
  bool scFlag; //image scale is set
  bool diFlag; //dither mode is set

public:
  
  /**
   * Fija espacio de color del archivo
   *
   *@param colorspace
   *@see bicotiColorSpace
   */ 
  void SetColorSpace( bicotiColorSpace colorspace );
  
  /**
   * Fija calidad de la imagen 
   * 
   *@param in_quality  indica porcentaje de caliad (1 a 100)
   *@param in_force_baseline true evita que se generen archivos de muy mala calidad sobre todo cuando la calidad est por debajo de 25.
   */
  void SetQuality(int in_quality, bool in_force_baseline);
  
  /**
   * Fija espacio de color de la imagen
   *
   *@param colorspace 
   *@see bicotiColorSpace 
   */ 
  void SetImageColorSpace(bicotiColorSpace colorspace);

  /**
   * Reduce la imagen a la escala indicada
   *
   *@param num numerador
   *@param den denominador
   */
  void SetScale(int num, int den);
  
  /**
   * Fija tratamiento de dither
   *
   *@param dither 
   *@see bicotiDitherMode
   */
  void SetDitherMode( bicotiDitherMode dither);
  
  /**
   * Fija valores por defecto
   *
   * espacio de color = rgb
   */
  void SetDefaults();

};

//  ========================== Implementations  =============================


template< class PixelType >
void bicotiMapperJPEG< PixelType > :: SetColorSpace(bicotiColorSpace colorspace)
{
  csFlag = true;
  file_colorspace = colorspace;
}


template< class PixelType >
void bicotiMapperJPEG< PixelType > :: SetImageColorSpace(bicotiColorSpace colorspace)
{
  image_colorspace = colorspace;
}

template< class PixelType >
void bicotiMapperJPEG< PixelType > :: SetQuality(int in_quality, bool in_force_baseline)
{
  qtFlag = true;
  quality = in_quality;
  force_baseline =  in_force_baseline;
}

template< class PixelType >
void bicotiMapperJPEG< PixelType > :: SetScale(int num, int den)
{
  scFlag = true;
  scale_num= num;
  scale_den = den;
}

template< class PixelType >
void bicotiMapperJPEG< PixelType > :: SetDitherMode(bicotiDitherMode dither)
{
  diFlag = true;
  dither_mode = dither;
}

template< class PixelType >
void bicotiMapperJPEG< PixelType > :: SetDefaults()
{
  csFlag = false;
  qtFlag = false;
  scFlag = false;
  diFlag = false;
  image_colorspace = cs_rgb;
}


#endif
